@@ -9,7 +9,7 @@
 #    Copyright (C) 1999 Francis J. Lacoste, iNsu Innovations
 #    Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
 #    Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
-#    Parts Copyright 2001-2003 by Mark Stosberg <mark@stosberg.com>
+#    Parts Copyright 2001-2002 by Mark Stosberg <mark@stosberg.com>
 #
 #    Parts of this module are based on work by
 #    Bruce Albrecht, <bruce.albrecht@seag.fingerhut.com> contributed to
@@ -20,11 +20,12 @@
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms same terms as perl itself.
 #
+#    $Header: /cvsroot/cascade/dfv/lib/Data/FormValidator.pm,v 1.8 2003/03/23 02:57:23 markjugg Exp $
 package Data::FormValidator;
 
 use vars qw( $VERSION $AUTOLOAD);
 
-$VERSION = '2.10.1';
+$VERSION = '2.01';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -455,11 +456,12 @@ value "postcode".
 
 =item untaint_all_constraints
 
-If this field is set all form data that passes a constraint will be untainted.
-The untainted data will be returned in the valid hash. Untainting is based on
-the pattern match used by the constraint. If you write your own regular
-expressions and only match part of the string then you'll only get part of the
-string in the valid hash. It is a good idea to write you own constraints like
+If this field is set all form data that passes a constraint will be
+untainted. The untainted data will be returned in the valid
+hash. Untainting is based on the pattern match used by the
+constraint. If you write your own regular expressions and only match
+part of the string then you'll only get part of the string in the
+valid hash. It is a good idea to write you own constraints like
 /^regex$/. That way you match the whole string.
 
 This is overridden by untaint_constraint_fields
@@ -467,130 +469,47 @@ This is overridden by untaint_constraint_fields
 =item untaint_constraint_fields
 
 Specifies that one or more fields will be untainted if they pass their
-constraint(s). This can be set to a single field name or an array reference of
-field names. The untainted data will be returned in the valid hash. Untainting
-is based on the pattern match used by the constraint. If you write your own
-regular expressions and only match part of the string then you'll only get part
-of the string in the valid hash. It is a good idea to write you own constraints
-like /^regex$/. That way you match the whole string.
+constraint(s). This can be set to a single field name or an array
+reference of field names. The untainted data will be returned in the
+valid hash. Untainting is based on the pattern match used by the
+constraint. If you write your own regular expressions and only match
+part of the string then you'll only get part of the string in the
+valid hash. It is a good idea to write you own constraints like
+/^regex$/. That way you match the whole string.
 
 This is overrides the untaint_all_constraints flag.
 
 =item missing_optional_valid
 
-This can be set to a true value (such as 1) to cause missing optional fields to
-be included in the valid hash. By default they are not included-- this is the
-historical behavior. 
+This can be set to a true value (such as 1) to cause missing optional
+fields to be included in the valid hash. By default they are not
+included-- this is the historical behavior. 
 
 =item validator_packages 
 
-This key is used to define other packages which contain validation routines.
+This key is used to define other packages which contain validation routines. 
 Set this key to a single package name, or an arrayref of several. All of its
-subs beginning with 'match_' and 'valid_' will be imported into
-Data::FormValidator.  This lets you reference them in a constraint with just
-their name (the part after the underscore).  You can even override the provided
-validators.
+subs beginning with 'match_' and 'valid_' will be imported into Data::FormValidator.
+This lets you reference them in a constraint with just their name (the part
+after the underscore).  You can even override the provided validators.
 
 B<Example>:
 
 	validator_packages => [qw(ProjectName::Validate::Basic)],
 
-=item msgs
-
-This key is used to transform the output of the C<$invalid> and C<$missing> 
-return values from array references to hash references which provide the field
-names as keys and error messages as values. By default, invalid fields have the message
-"Invalid" associated with them while missing fields have the message "Missing"
-associated with them. In the simplest case, this key can simply be defined as 
-a reference to a an empty hash, like this:
-
-B<Example>:
-
-	msgs =>{}
-
-This will cause the default messages to be used for missing and invalid fields. Some
-default formatting will also be applied, designed for display in an XHTML web
-page. That formatting is as followings:
-
-	<span style="color:red;font-weight:bold"><span id="dfv_errors">* %s</span></span>
-
-The C<%s> will be replaced with the message. The effect is that the message
-will appear in bold red with an asterisk before it. This style can be overriden by simply
-defining "dfv_errors" appropriately in a style sheet, or by providing a new format string.
-
-Here's a more complex example that shows how to provide your own default message strings, as well
-as providing custom messages per field, and with multiple constraints:
-
-	msgs => {
-	   invalid => {
-
-		   # define custom messages for invalid fields
-		   field => {
-				   first_name => 'may contain only alphabet letters',
-				   phone => 'may contain only numeric characters',
-
-				   # custom messages for an invalid field with multiple constraints
-				   age => {
-						   min => 'must be at least 18 years old',
-						   max => 'how can he be older than 150?'
-				   },
-		   },
-
-		   # a default message for invalid fields 
-		   default => 'contains an invalid value'
-	   },
-
-	   # You can define per-field messages for missing fields, too
-	   # Here, only a new default message is used. 
-	   missing => {
-		 default=> 'contains a missing value '
-	   },
-
-	   # providing a custom formatting string
-	   format => 'ERROR: %s',
-
-	   # Prefix all field names with this string.
-	   # Setting prefix to the value of '',
-	   # causing the default string 'err_' to be used.
-	   prefix => 'error_', 
-	}
-
-
-
-Here's an example of an <$invalid> hash reference that's returned with
-messages. Fields with multiple constraints are returned in a style that's
-directly compatible with HTML::Template.
-
-	$invalid =  {
-		field_with_single_constraints 		=> 'message',
-		field_with_multiple_constraints 	=> [
-			{   
-				constraint => 'min',
-				msg 	   => 'not high enough', 	
-			},
-			{   
-				constraint => 'max',
-				msg 	   => 'not low enough', 	
-			},
-		],
-	};
-
-
-
-
 =back
 
 =head1 VALIDATING INPUT BASED ON MULTIPLE FIELDS
 
-You can pass more than one value into a validation routine.  For that, the
-value of the constraint should be a hash reference. One key should named
-C<constraint> and should have a value set to the reference of the subroutine or
-the name of a built-in validator.  Another required key is I<params>. The value
-of the I<params> key is a reference to an array of the other elements to use in
-the validation. If the element is a scalar, it is assumed to a field name. If
-the value is a reference, what the reference points to is passed into the
-subroutine.  (Don't forget to include the name of the field to check in that
-list!)
+You can pass more than one value into a validation routine. 
+For that, the value of the constraint should be a 
+a hash reference. One key should named C<constraint> and should have a value 
+set to the reference of the subroutine or the name of a built-in validator.
+Another required key is I<params>. The value of the I<params> key is a
+reference to an array of the other elements to use in the validation. If the
+element is a scalar, it is assumed to a field name. If the value is a reference,
+what the reference points to is passed into the subroutine. 
+(Don't forget to include the name of the field to check in that list!)
 
 B<Example>:
 
@@ -602,9 +521,9 @@ B<Example>:
 
 =head1 MULTIPLE CONSTRAINTS
 
-Multiple constraints can be applied to a single field by defining the value of
-the constraint to be an array reference. Each of the values in this array can
-be one of the constraint types defined above: the name of a built-in validator,
+Multiple constraints can be applied to a single field by defining
+the value of the constraint to be an array reference. Each of the values in this array
+can be one of the constraint types defined above: the name of a built-in validator, 
 a regular expression, or a subroutine reference. 
 
 It's important to know which of the constraints failed, so fields defined
@@ -735,7 +654,7 @@ sub validate {
 
     # import valid_* subs from requested packages
 	foreach my $package (_arrayify($profile->{validator_packages})) {
-		if ( !exists $profile->{imported_validators}{$package} ) {
+		if ( !exists $self->{imported_validators}{$package} ) {
 			eval "require $package";
 			if ($@) {
 				die "Couldn't load validator package '$package': $@";
@@ -749,7 +668,7 @@ sub validate {
 					*{qualify_to_ref($sub)} = $subref;
 				}
 			}
-			$profile->{imported_validators}{$package} = 1;
+			$self->{imported_validators}{$package} = 1;
 		}
 	}
 
@@ -802,17 +721,11 @@ sub validate {
     my %optional    = map { $_ => 1 } _arrayify($profile->{optional});
 
     # loop through and add fields to %required and %optional based on regular expressions   
-    my $required_re;
-    if ($profile->{required_regexp}) {
-	$required_re = eval 'sub { $_[0] =~ '. $profile->{required_regexp} . '}';
-	die "Error compiling regular expression $required_re: $@" if $@;
-    }
+    my $required_re = eval 'sub { $_[0] =~ '. $profile->{required_regexp} . '}' if $profile->{required_regexp};
+    die "Error compiling regular expression $required_re: $@" if $@;
 
-    my $optional_re;
-    if ($profile->{optional_regexp}) {
-	$optional_re = eval 'sub { $_[0] =~ '. $profile->{optional_regexp} . '}';
-	die "Error compiling regular expression $optional_re: $@" if $@;
-    }
+    my $optional_re = eval 'sub { $_[0] =~ '. $profile->{optional_regexp} . '}' if $profile->{optional_regexp};
+    die "Error compiling regular expression $optional_re: $@" if $@;
 
     foreach my $k (keys %valid) {
        if ($required_re && $required_re->($k)) {
@@ -838,7 +751,7 @@ sub validate {
 		if (ref $valid{$field}) {
 			if ( ref $valid{$field} eq 'ARRAY' ) {
 				for (my $i = 0; $i < scalar @{ $valid{$field} }; $i++) {
-					$valid{$field}->[$i] = undef unless length $valid{$field}->[$i];
+					delete $valid{$field}->[$i] unless length $valid{$field}->[$i];
 				}
 			}
 		}
@@ -949,10 +862,10 @@ sub validate {
 	   foreach my $constraint_spec (_arrayify($constraint_list)) {
 			 my $c = _constraint_hash_build($field,$constraint_spec,$untaint_this);
 
-			 my $is_value_list = 1 if (ref $valid{$field} eq 'ARRAY');
+			 my $is_value_list = 1 if ref $valid{$field};
 			 if ($is_value_list) {
 				 foreach (my $i = 0; $i < scalar @{ $valid{$field}} ; $i++) {
-					 my @params = _constraint_input_build($c,$valid{$field}->[$i],\%valid);
+					 my @params = _constraint_input_build($c,$valid{$field}->[$i]);
 
 					 my ($match,$failed) = _constraint_check_match($c,\@params);
 					 if ($failed) {
@@ -964,7 +877,7 @@ sub validate {
 				 }
 			 }
 			 else {
-				my @params = _constraint_input_build($c,$valid{$field},\%valid);
+				my @params = _constraint_input_build($c,$valid{$field});
 				my ($match,$failed) = _constraint_check_match($c,\@params);
 				if ($failed) {
 					push @invalid_list, $failed
@@ -995,91 +908,9 @@ sub validate {
 		}
 	}
 
-	my ($missing,$invalid);
-	if (ref $profile->{msgs} eq 'HASH') {
-		my $msgs = $profile->{msgs};
-		$invalid = _msg_hash_create($msgs->{invalid},$msgs->{format},\@invalid,'Invalid');
-		$missing = _msg_hash_create($msgs->{missing},$msgs->{format},\@missings,'Missing');
-
-		if (defined $msgs->{prefix}) {
-			my $pre = ($msgs->{prefix} eq '') ? 'err_' : $msgs->{prefix};
-			$invalid = prefix_hash($pre,$invalid);
-			$missing = prefix_hash($pre,$missing);
-		}
-	}
-	else {
-		$missing = \@missings;
-		$invalid = \@invalid;
-	}
-
-    return ( \%valid, $missing, $invalid, \@unknown );
+    return ( \%valid, \@missings, \@invalid, \@unknown );
 }
 
-# INPUT: prefix_string, hash reference
-# Copies the hash and prefixes all keys with prefix_string
-# OUTPUT: hash refence
-sub prefix_hash {
-	my ($pre,$href) = @_;
-	die "prefix_hash: need two arguments" unless (scalar @_ == 2);
-	die "prefix_hash: second argument must be a hash ref" unless (ref $href eq 'HASH');
-	my %out; 
-	for (keys %$href) {
-		$out{$pre.$_} = $href->{$_};
-	}
-	return \%out;
-}
-
-
-# If the msgs key is used in the profile, we transform the output data error structures
-# into hash references which contain error messages.
-sub _msg_hash_create {
-	my ($in_hash,$fmt,$array,$default) = @_;
-	die '_msg_hash_create: expecting 4 input arguments' unless (scalar @_ == 4);
-	my %msgs;
-	$default = $in_hash->{default} if defined $in_hash->{default};
-
-	for my $f (@$array) {
-		my $f_msgs = $in_hash->{field} || {};
-
-		my $field_name;
-		# If we have multiple constraints (only applies to the "invalid" case)
-		if (ref $f eq 'ARRAY') {
-			$field_name = $f->[0];
-
-			for (my $i = 1; $i < @$f; $i++) {
-				my $constraint = $f->[$i];
-
-				# If we have multiple messages
-				if (ref $f_msgs->{$field_name} eq 'HASH') {
-					my $msg = ($f_msgs->{$field_name}{$constraint} || $default);
-					push @{ $msgs{$field_name} }, {
-						constraint => $constraint,
-						msg => _error_msg_fmt($fmt,$msg),
-					};
-				}
-			}
-		}
-		else {
-			$field_name = $f;
-		}
-
-		$msgs{$field_name} ||= (defined $f_msgs->{$field_name} ? $f_msgs->{$field_name} : $default);
-		$msgs{$field_name} = _error_msg_fmt($fmt,$msgs{$field_name}) unless (ref $msgs{$field_name} eq 'ARRAY');
-	}
-
-	return \%msgs;
-}
-
-sub _error_msg_fmt ($$) {
-	my ($fmt,$msg) = @_;
-	$fmt ||= 
-			'<span style="color:red;font-weight:bold"><span id="vrm_errors">* %s</span></span>';
-	($fmt =~ m/%s/) || die 'format must contain %s'; 
-	return sprintf $fmt, $msg;
-}
-
-
-
 # takes string or array ref as input
 # returns array
 sub _arrayify {
@@ -1729,24 +1560,23 @@ sub _check_profile_syntax {
 	die "Invalid input profile: needs to be a hash reference\n" unless ref $profile eq "HASH";
 
 	my @valid_profile_keys = (qw/
-		constraint_regexp_map
-		constraints
-		defaults
-		dependencies
-		dependency_groups
-		field_filter_regexp_map
-		field_filters
-		filters
-		missing_optional_valid
-		msgs
 		optional
-		optional_regexp
-		require_some
 		required
 		required_regexp 
-		untaint_all_constraints
+		require_some
+		optional_regexp
+		constraints
+		constraint_regexp_map
+		dependencies
+		dependency_groups
+		defaults
+		filters
+		field_filters
+		field_filter_regexp_map
+		missing_optional_valid
 		validator_packages
         untaint_constraint_fields
+		untaint_all_constraints
 		/);
 
 	# If any of the keys in the profile are not listed as valid keys here, we die with an error	
@@ -1847,14 +1677,14 @@ sub _constraint_hash_build {
 }
 
 sub _constraint_input_build {
-	my ($c,$value,$valid) = @_;
-	die "_constraint_input_build recieved wrong number of arguments" unless (scalar @_ == 3);
+	my ($c,$value) = @_;
+	die "_constraint_input_build recieved wrong number of arguments" unless (scalar @_ == 2);
 
 	my @params;
 	if (defined $c->{params}) {
 		foreach my $fname (_arrayify($c->{params})) {
 			# If the value is passed by reference, we treat it literally
-			push @params, (ref $fname) ? $fname : $valid->{$fname}
+			push @params, (ref $fname) ? $fname : $value 
 		}
 	}
 	else {
